// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

#include "dictionary.h"

#include "KVpair.h"

// Dictionary implemented with an unsorted array-based list
template <typename Key, typename Elem>
class UALdict : public Dictionary<Key, Elem> {
private:
  AList<KVpair<Key,Elem> >* list;
public:
  UALdict(int size=DefaultListSize)   // Constructor
    { list = new AList<KVpair<Key,Elem> >(size); }
  ~UALdict() { delete list; }         // Destructor
  void clear() { list->clear(); }     // Reinitialize

  // Insert an element: append to list
  void insert(const Key&k, const Elem& e) {
    KVpair<Key,Elem> temp(k, e);
    list->append(temp);
  }

  // Use sequential search to find the element to remove
  bool remove(const Key& K, Elem& e) {
    if(find(K, e)) { // Use "find" to set list position
      KVpair<Key,Elem> temp = list->remove();
      return true;
    }
    return false;
  }

  Elem removeAny() { // Remove the last element
    Assert(size() != 0, "Dictionary is empty");
    list->movetoEnd();
    list->prev();
    KVpair<Key,Elem> e = list->remove();
    return e.value();
  }

  // Find "K" using sequential search
  bool find(const Key& K, Elem& e) const {
    for(list->movetoStart(); list->rightLength() > 0;
        list->next()) {
      KVpair<Key,Elem> temp = list->getValue();
      if (K == temp.key()) {
        e = temp.value();
        return true;
      }
    }
    return false; // "K" does not appear in dictionary
  }
  int size() // Return list size
    { return list->leftLength() + list->rightLength(); }
};
