// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// Improved ADT for buffer pools using the buffer-passing
// style. Most user functionality is in the buffer class,
// not the buffer pool itself.

// A single buffer in the buffer pool
class Buffer {
public:
  // Read the associated block from disk (if necessary) and
  // return a pointer to the data
  void *readBlock() = 0;

  // Return a pointer to the buffer's data array
  // (without reading from disk)
  void *getDataPointer() = 0;

  // Flag the buffer's contents as having changed, so that
  // flushing the block will write it back to disk
  void markDirty() = 0;

  // Release the block's access to this buffer. Further
  // accesses to this buffer are illegal.
  void releaseBuffer() = 0;
}

// The bufferpool
class BufferPool {
public:
  // Constructor: This bufferpool has "numbuff" buffers that
  // each contain "buffsize" bytes of data.
  BufferPool(int numbuff, int buffsize) = 0;

  // Relate a block to a buffer, returning a pointer to a
  // buffer object
  Buffer *acquireBuffer(int block) = 0;
}
