// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// Doubly linked list link node with freelist support
template <typename Elem> class Link {
private:
  static Link<Elem>* freelist; // Reference to freelist head

public:
  Elem element;       // Value for this node
  Link* next;         // Pointer to next node in list
  Link* prev;         // Pointer to previous node

  // Constructors
  Link(const Elem& e, Link* prevp, Link* nextp) {
    element = e;
    prev = prevp;
    if (prev != NULL) prev->next = this;
    next = nextp;
    if (next != NULL) next-> prev = this;
  }
  Link(Link* prevp =NULL, Link* nextp =NULL) {
    prev = prevp;
    if (prev != NULL) prev->next = this;
    next = nextp;
    if (next != NULL) next-> prev = this;
  }

  void* operator new(size_t) {  // Overloaded new operator
    if (freelist == NULL) return ::new Link; // Create space
    Link<Elem>* temp = freelist; // Can take from freelist
    freelist = freelist->next;
    return temp;                 // Return the link
  }

  // Overloaded delete operator
  void operator delete(void* ptr) {
    ((Link<Elem>*)ptr)->next = freelist; // Put on freelist
    freelist = (Link<Elem>*)ptr;
  }
};

// The freelist head pointer is actually created here
template <typename Elem>
Link<Elem>* Link<Elem>::freelist = NULL;
