// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// This is the file to include in your code if you want access to the
// complete LList template class

// First, get the declaration for the base list class
#include "list.h"

// This is the declaration for LList. It is split into two parts
// because it is too big to fit on one book page
// Linked list implementation
template <typename Elem> class LList: public List<Elem> {
private:
  Link<Elem>* head;       // Pointer to list header
  Link<Elem>* tail;       // Pointer to last Elem in list 
  Link<Elem>* fence;      // Last element on left side
  int leftcnt;            // Size of left partition
  int rightcnt;           // Size of right partition

  void init() {           // Intialization helper method
    fence = tail = head = new Link<Elem>;
    leftcnt = rightcnt = 0;
  }

  void removeall() {   // Return link nodes to free store
    while(head != NULL) {
      fence = head;
      head = head->next;
      delete fence;
    }
  }

public:
  LList(int size=DefaultListSize) { init(); } // Constructor
  ~LList() { removeall(); }                   // Destructor
  void print() const;                // Print list contents
  void clear() { removeall(); init(); }       // Clear list

  // Insert "it" at front of right partition
  bool insert(const Elem& it) {
    fence->next = new Link<Elem>(it, fence->next);  
    if (tail == fence) tail = fence->next;  // New tail
    rightcnt++;
    return true;
  }

  bool append(const Elem& it) { // Append "it" to list
    tail = tail->next = new Link<Elem>(it, NULL);
    rightcnt++;
    return true;
  }

  void movetoStart() // Place fence at list start
    { fence = head; rightcnt += leftcnt; leftcnt = 0; }

  void movetoEnd()   // Place fence at list end
    { fence = tail; leftcnt += rightcnt; rightcnt = 0; }
  // Remove and return first Elem in right partition
  Elem remove() {
    Assert(rightLength() > 0, "Nothing to remove");
    Elem it = fence->next->element;  // Remember value
    Link<Elem>* ltemp = fence->next; // Remember link node
    fence->next = ltemp->next;       // Remove from list
    if (tail == ltemp) tail = fence; // Reset tail
    delete ltemp;                    // Reclaim space
    rightcnt--;			     // Decrement the count
    return it;
  }

  // Move fence one step left; no change if left is empty
  void prev() {
    if (fence == head) return;       // No previous Elem
    Link<Elem>* temp = head;
    // March down list until we find the previous element
    while (temp->next!=fence) temp=temp->next;
    fence = temp;
    leftcnt--; rightcnt++;           // Set the counts
  }

  // Move fence one step right; no change if right is empty
  void next() {
    if (fence != tail)
      { fence = fence->next; rightcnt--; leftcnt++; }
  }

  int leftLength() const  { return leftcnt; }
  int rightLength() const { return rightcnt; }

  // Set the size of left partition to "pos"
  void movetoPos(int pos) {
    Assert ((pos>=0)&&(pos<=rightcnt+leftcnt),
            "Position out of range");
    rightcnt = rightcnt + leftcnt - pos; // Set counts
    leftcnt = pos;
    fence = head;
    for(int i=0; i<pos; i++) fence = fence->next;
  }

  const Elem& getValue() const { // Return current Elem
    Assert(rightLength() > 0, "Nothing to get");
    return fence->next->element;
  }
};
